/*
 * lpcm.c:	Use the S/P-DIF interface for redirecting PCM stream
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 *
 * Copyright (C) 2003 Werner Fink, <werner@suse.de>
 */

#include "types.h"
#include "lpcm.h"

static frame_t & scan_pcm_frame(const uint_8 *&out, const uint_8 *const tail);
iec60958 pcm(scan_pcm_frame, 48000, SPDIF_BURST_SIZE, 0);

# define debug_pcm(args...)
// # define debug_pcm(args...)	fprintf(stderr, args)

// This function requires two arguments:
//   first is the start of the data segment
//   second is the tail of the data segment
static frame_t & scan_pcm_frame(const uint_8 *&out, const uint_8 *const tail)
{
    static size_t pos = 0;
    // We overwrite the none audio magic PCM header (offset is zero)
    const static size_t payload_size = SPDIF_BURST_SIZE;

    debug_pcm("<<<<<<<< pcm scan %d -> \n", tail-out);

    pcm.play.burst = (uint_32 *)0;
    pcm.play.size = 0;

    while(pos < payload_size) {
	int rest = tail - out;
	if (rest <= 0)
	{
	    debug_pcm("XXXXXXXX rest=%d, pos=%d\n", rest, pos);
	    goto done;
	}
        debug_pcm("\t rest=%d, pos=%d\n", rest, pos);
	if (rest + pos > payload_size)
	    rest = payload_size - pos;
	memcpy(&pcm.current[pos], out, rest);
	pos += rest;
	out += rest;
    }
    swab(pcm.payload, pcm.payload, payload_size);

    pcm.play.burst = (uint_32 *)(&pcm.current[0]);
    pcm.play.size = payload_size;
    pcm.play.pay = payload_size;

    pcm.Switch();

    pcm.last.burst = (uint_32 *)(&pcm.remember[0]);
    pcm.last.size = pcm.play.size;
    pcm.last.pay = pcm.play.pay;

    debug_pcm(">>>>>>>> %d, pos=%d\n", tail-out, pos);

    pos = 0;
done:
    return pcm.play;
}
